@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Ported Perl from the book's C code
 
-our $VERSION = '0.0200';
+our $VERSION = '0.0300';
 
 use Moo;
 use strictures 2;
@@ -436,6 +436,38 @@ sub permi {
 }
 
 
+sub pfold {
+    my ($self, $n, $m, $f) = @_;
+    my @sequence;
+    my ($j, $k);
+    for (my $i = 1; $i <= $n; ++$i) {
+      _oddeven($i, \$k, \$j);
+      $k = $k % $m;
+      my $y = $f & (1 << $k) ? 1 : 0;
+      if ((2 * $j + 1) % 4 > 1) {
+          $y = 1 - $y;
+      }
+      push @sequence, $y;
+    }
+    return \@sequence;
+}
+
+# find x and y such that n = 2^x * (2*y+1)
+sub _oddeven {
+    my ($n, $x, $y) = @_;
+    my $k;
+    # two's complement of n = -n or ~n + 1
+    my $l = $n & -$n; # this is 2^a
+    $$y = ($n / $l - 1) / 2;
+    for ($k = 0; $l > 1; ++$k) {
+        $l >>= 1;
+    }
+    $$x = $k;
+
+    return;
+}
+
+
 sub reverse_at {
     my ($self, $n, $parts) = @_;
     my @head = @$parts[ 0 .. $n - 1 ];
@@ -471,7 +503,7 @@ Music::CreatingRhythms - Ported Perl from the book's C code
 
 =head1 VERSION
 
-version 0.0200
+version 0.0300
 
 =head1 SYNOPSIS
 
@@ -641,6 +673,12 @@ array-reference of array-references.
 (For an efficient iterator, check out the L<Algorithm::Combinatorics>
 module.)
 
+=head2 pfold
+
+  $sequences = $mcr->pfold($n, $m, $f);
+
+Generate fold sequences.
+
 =head2 reverse_at
 
   $sequence = $mcr->reverse_at($n, $parts);
